#!/usr/bin/env python3

import math
import collections

LAVA_SPOT_COUNT =  6
PLAYER_COUNT    =  2
ENEMY_COUNT     =  4
AMMO            = 10

Size = collections.namedtuple('Size', ('width', 'height', 'depth'))

class Point:

    def __init__(self, x=0, y=0, z=0):
        self.x = x
        self.y = y
        self.z = z

    @staticmethod
    def random(min, max):
        return Point(math.rand_range(min.x, max.x + 1),
                     math.rand_range(min.y, max.y + 1),
                     math.rand_range(min.z, max.z + 1))

    def __eq__(self, other):
        return self.x == other.x and self.y == other.y and self.z == other.z

class Enemy:

    def __init__(self, position):
        self.position = position

    @staticmethod
    def at_random_position(min, max):
        return Enemy(Point.random(min, max))

class Player:

    def __init__(self, position, ammo):
        self.position = position
        self.ammo = ammo

    @staticmethod
    def at_random_position(min, max, ammo):
        return Player(Point.random(min, max), ammo)

    def move(self):
        self.position.x += 14

    def shoot(self):
        self.ammo -= 1

class Map:

    def __init__(self, size, lava_spots):
        self.size = size
        self.lava_spots = lava_spots

    def in_lava(self, point):
        return point in self.lava_spots

class Game:

    def __init__(self, map, players=None, enemies=None):
        self.map = map
        self.players = players or []
        self.enemies = enemies or []

    def add_player(self, player):
        self.players.append(player)

    def add_enemy(self, enemy):
        self.enemies.append(enemy)

    def tick(self):
        i = 0

        while True:
            j = 0

            if i >= len(self.players):
                break

            self.players[i].move()

            if self.map.in_lava(self.players[i]):
                print("Player %u fell in the lava!".format(i + 1))
                self.players.remove(i)
                continue

            while True:
                if j >= len(self.enemies):
                    break

                if self.enemies[j].position == self.players[i].position:
                    print("Player %u shot enemy {}!".format(i + 1, j + 1))
                    self.enemies.remove(j)
                    continue

                j += 1

            i += 1

def main():
    map_size = Size(20, 1, 20)
    zero = Point()
    max = Point(map_size.width - 1, map_size.height - 1, map_size.depth - 1)
    game = Game()
    lava_spots = [Point.random() for x in range(LAVA_SPOT_COUNT)]

    game.set_map(Map(Size(w, h, d), lava_spots)

    for n in range(PLAYER_COUNT):
        game.add_player(Player.at_random_position(zero, max, AMMO))

    for n in range(ENEMY_COUNT):
        game.add_enemy(Enemy.at_random_position(zero, max))

    while len(game.players) > 0 and len(game.enemies) > 0:
        game.tick()

    if len(game.players) <= 0:
        print('All players died, enemies win :(')
    else:
        print('All enemies died, players win :)')

main()

# vi: set et sw=4 ts=4 tw=79:

